@@ -45,13 +45,16 @@ package Net::BitTorrent::Peer;
         predicate  => '_has_pieces',
         writer     => '_set_pieces',
         clearer    => '_clear_pieces',
-        trigger    => \&_trigger_pieces,
+        trigger    => sub { shift->_trigger_pieces },
         handles    => {
             _set_piece      => 'Bit_On',
             _has_piece      => 'bit_test',
             seed            => 'is_full',
             _check_interest => sub {
                 my $s = shift;
+
+                #return $s->_unset_interesting if $s->is_seed;
+                #warn $s->_wanted_pieces->to_Enum;
                 $s->_wanted_pieces->Norm
                     ? $s->_set_interesting
                     : $s->_unset_interesting;
@@ -75,9 +78,66 @@ package Net::BitTorrent::Peer;
             if $o && !$s->local_connection;
         return if !$s->_has_torrent;
         $s->pieces->Resize($s->torrent->piece_count);
-        $s->_check_interest;
     }
-    after '_set_piece' => \&_check_interest;
+    after '_set_piece' => sub { shift->_check_interest };
+
+    #
+    has 'remote_requests' => (is      => 'ro',
+                              isa     => 'ArrayRef[ArrayRef]',
+                              traits  => ['Array'],
+                              handles => {_add_remote_request    => 'push',
+                                          _shift_remote_requests => 'shift',
+                                          _clear_remote_requests => 'clear',
+                                          _count_remote_requests => 'count'
+                              },
+                              default => sub { [] }
+    );
+    has 'requests' => (
+        is => 'ro',
+        isa =>
+            'ArrayRef[Net::BitTorrent::Protocol::BEP03::Metadata::Piece::Block]',
+        traits  => ['Array'],
+        handles => {_add_request    => 'push',
+                    _clear_requests => 'clear',
+                    _count_requests => 'count',
+                    _first_request  => 'first',
+                    _delete_request => 'delete'
+        },
+        default => sub { [] }
+    );
+    around '_delete_request' => sub {
+        my ($c, $s, $i, $o, $l) = @_;
+        return $c->($s, $i) if !blessed $i;
+        my $x = 0;
+        $s->_find_request(
+            sub {
+                $x++;
+                $_->index == $i && $_->offset == $o && $_->length == $l;
+            }
+        );
+        return $c->($s, $x);
+    };
+
+    sub _find_request {
+        my ($s, $i, $o, $l) = @_;
+        my $x = 0;
+        my $p = $s->_first_request(
+            sub {
+                $x++;
+                $_->index == $i && $_->offset == $o && $_->length == $l;
+            }
+        );
+        wantarray ? [$p, $x] : $p;
+    }
+    around '_add_request' => sub {
+        my ($c, $s, $b) = @_;
+        return if $s->remote_choked;
+        $c->($s, $b);    # XXX - also let the parent client know
+        require Scalar::Util;
+        Scalar::Util::weaken($s->requests->[-1]);
+        $s->_send_request($b);
+        $b->_set_peer($s);
+    };
 
     #
     has 'peer_id' => (isa       => 'NBTypes::Client::PeerID',
@@ -86,6 +146,19 @@ package Net::BitTorrent::Peer;
                       predicate => '_has_peer_id'
     );
 
+    #
+    has 'quests' => (is      => 'ro',
+                     isa     => 'HashRef[Defined]',
+                     traits  => ['Hash'],
+                     handles => {_add_quest    => 'set',
+                                 _get_quest    => 'get',
+                                 _has_quest    => 'defined',
+                                 _delete_quest => 'delete',
+                                 _clear_quests => 'clear'
+                     },
+                     default => sub { {} }
+    );
+
     #
     for my $flag (
         ([0,
@@ -108,6 +181,8 @@ package Net::BitTorrent::Peer;
                    }
         ) for @{$flag->[1]};
     }
+
+    #
     around '_set_interesting' => sub {
         my ($c, $s) = @_;
         return if $s->interesting;
@@ -141,13 +216,16 @@ package Net::BitTorrent::Peer;
     );
 
     # Utility methods
-    sub _check_unique_connection {    # XXX - Rename this method
-                                      #return;
-        my ($s) = @_;
-        return
-            if scalar(grep { $_->_has_peer_id && $_->peer_id eq $s->peer_id }
-                          $s->torrent->peers
-            ) <= 1;
+    sub _check_unique_connection {
+        my $s = shift;
+        return 1
+            if scalar(
+            grep {
+                       $_->_has_peer_id
+                    && $_->_id ne $s->_id
+                    && $_->peer_id eq $s->peer_id
+                } $s->torrent->peers
+            ) == 0;
         $s->disconnect(sprintf '%s already has connection for this torrent',
                        $s->peer_id);
     }
@@ -171,6 +249,9 @@ package Net::BitTorrent::Peer;
         $_[0] = undef;
     }
 
+    sub DEMOLISH {
+    }
+
     #
     no Moose;
     __PACKAGE__->meta->make_immutable;
@@ -443,70 +524,6 @@ The time since any transfer occurred with this peer.
         $s->handshake_step;
     };
 
-    has 'remote_requests' => (is      => 'ro',
-                              isa     => 'ArrayRef[ArrayRef]',
-                              traits  => ['Array'],
-                              handles => {_add_remote_request    => 'push',
-                                          _shift_remote_requests => 'shift',
-                                          _clear_remote_requests => 'clear',
-                                          _count_remote_requests => 'count'
-                              },
-                              default => sub { [] }
-    );
-    has 'requests' => (
-                    is  => 'ro',
-                    isa => 'ArrayRef[Net::BitTorrent::Torrent::Piece::Block]',
-                    traits  => ['Array'],
-                    handles => {_add_request    => 'push',
-                                _clear_requests => 'clear',
-                                _count_requests => 'count'
-                    },
-                    default => sub { [] }
-    );
-    around '_add_request' => sub {
-        my ($c, $s, $b) = @_;
-        return if $s->choked;
-        $c->($s, $b);    # XXX - also let the parent client know
-
-        # XXX - also let the parent client know
-        $s->_send_request($b);
-        $b->_set_peer($s);
-    };
-    around '_unset_remote_choked' => sub {
-        my ($c, $s) = @_;
-        return if $s->_has_quest('request_block');
-        require Scalar::Util;
-        Scalar::Util::weaken $s;
-        my $max_requests = 4;    # XXX - max_requests attribute
-        $s->_add_quest(
-            'request_block',
-            AE::timer(
-                0, 3,
-                sub {
-                    return if !defined $s;
-                    for (0 .. $max_requests) {
-                        my $piece = $s->torrent->select_piece($s);
-                        next if !$piece;
-                        my $b = $piece->_first_unassigned_block();
-                        next if !$b;
-                        $s->_add_request($b);
-                    }
-                }
-            )
-        );
-    };
-    has 'quests' => (is      => 'ro',
-                     isa     => 'HashRef[ArrayRef]',
-                     traits  => ['Hash'],
-                     handles => {_add_quest    => 'set',
-                                 _get_quest    => 'get',
-                                 _has_quest    => 'defined',
-                                 _delete_quest => 'delete',
-                                 _clear_quests => 'clear'
-                     },
-                     default => sub { {} }
-    );
-
     #
     my $infohash_constraint;
     after 'BUILDALL' => sub {
@@ -921,7 +938,7 @@ L<clarification of the CCA-SA3.0|http://creativecommons.org/licenses/by-sa/3.0/u
 Neither this module nor the L<Author|/Author> is affiliated with BitTorrent,
 Inc.
 
-=for rcs $Id: Peer.pm 4a832ab 2010-08-21 17:00:23Z sanko@cpan.org $
+=for rcs $Id: Peer.pm 62b27d0 2010-09-05 04:22:11Z sanko@cpan.org $
 
 
 =cut
